



#include "RHB_Custom.h"

#include "RHV.h"
#include "FBooklet.h"
//
#include "TCanvas.h"
#include "TStyle.h"



ClassImp (RHB_Custom)



//////////////////////////////
//
//  RHV2 local class
//

class RHV2: public RHV {

  private :

    class FBooklet2: public FBooklet {
      public :
        Int_t GetCurrentPageNum () {
          return this->currentPage;
        }
    };

  public :

    static const Char_t *GetCurrentPageName () {
      FBooklet2 *FB      = (FBooklet2*)RHV::currentRHV->currentBooklet;
      Int_t     page_num = FB->GetCurrentPageNum ();
      return FB->GetPageName (page_num);
    }

    static TCanvas *GetCurrentCanvas () {
      return RHV::currentRHV->currentCanvas;
    }

};



////////////////////////////////////
//
//  RHB_Custom
//

RHB_Custom::RHB_Custom (void) {
  this->initialized = kFALSE;
}

Int_t RHB_Custom::Start (void) {

  //  Initialisation du Custom après conf officielle
  //    - la surcharge de InitParams est Ok (on peut appeller la classe mère avant ou après dans la surcharge)
  //    - NOK pour InitHistos (pas de surcharge) et arrive avant la création des histos officiels de la classe mère
  //
  //    => surcharge de Start pour être sur d'arriver après les confs
  //
  //
  //    => on a du GetParam et du GetHisto mais pas de GetPad
  //
  if (!this->initialized) {
    this->ucn1_t     = GetParam ("N1_UCN_COUNT_t_sec");
    this->last_clock = 0;
    //   CUSTOMISATION des TProfiles "chX calc & rate"
    TH1 *prof;
    char str [256];
    for (int i=1; i<21; i++) {
      sprintf (str, "CH%d_CALC_profile", i);
      prof = this->GetHisto (str);
      prof->SetOption ("HIST");
      sprintf (str, "CH%d_RATE_profile", i);
      prof = this->GetHisto (str);
      prof->SetOption ("HIST");
    }
    //   CUSTOMISATION des TProfiles "ucn & gamma calc rate"
    this->ratio_clock = RATIO_DELAY;
    this->h_ucn1      = this->GetHisto ("N1_UCN_RATE_profile");
    this->h_ucn2      = this->GetHisto ("N2_UCN_RATE_profile");
    this->h_gam1      = this->GetHisto ("N1_GAM_RATE_profile");
    this->h_gam2      = this->GetHisto ("N2_GAM_RATE_profile");
    this->h_ratio     = this->GetHisto ("RATIO_UCN_profile");
    this->h_ucn1->SetLineColor (kBlue);
    this->h_ucn1->SetOption    ("HIST");
    this->h_ucn2->SetLineColor (kBlue);
    this->h_ucn2->SetOption    ("HIST");
    this->h_gam1->SetLineColor (kRed);
    this->h_gam1->SetOption    ("HIST");
    this->h_gam2->SetLineColor (kRed);
    this->h_gam2->SetOption    ("HIST");
    this->h_ratio->SetOption   ("HIST");
    this->h_num       = (TH1*) this->h_ratio->Clone ();
    this->h_div       = (TH1*) this->h_ratio->Clone ();
    this->initialized = kTRUE;
  }
  return FEventProcessorThread::Start ();
}


void RHB_Custom::SetParamValues (void) {
  FEventProcessorThread::SetParamValues ();
  if (this->ucn1_t->IsOn ()) {
    Double_t cur_clock = this->ucn1_t->GetValue ();
    if (cur_clock > this->ratio_clock) {
      //  ucn ratio refresh
      this->ratio_clock += RATIO_DELAY;
      this->h_ratio->Reset  ();
      this->h_num->Reset    ();
      this->h_div->Reset    ();
      this->h_num->Add      (this->h_ucn2);
      this->h_num->Add      (this->h_ucn1, -1);
      this->h_div->Add      (this->h_ucn2);
      this->h_div->Add      (this->h_ucn1, +1);
      this->h_ratio->Add    (this->h_num);
      this->h_ratio->Divide (this->h_div);
    }
    if (cur_clock < this->last_clock) {
      //  new cycle : ucn graph reset
      this->h_ucn1->Reset  ();
      this->h_gam1->Reset  ();
      this->h_ucn2->Reset  ();
      this->h_gam2->Reset  ();
      this->h_ratio->Reset ();
      this->ratio_clock = RATIO_DELAY;
      //  gamma histo -> 'same' ucn histo
      if (strcmp (RHV2::GetCurrentPageName (), "UCNs") == 0) {
        TCanvas *curcav = RHV2::GetCurrentCanvas ();
        curcav->cd ();
        curcav->cd (3);
        this->h_gam1->Draw ("same");
        curcav->cd (4);
        this->h_gam2->Draw ("same");
        curcav->cd ();
        curcav->Update ();
      }
    }
    this->last_clock = cur_clock;
  }
}

