#!/usr/bin/python
# -*- encoding: utf8 -*-

import sys
sys.path.append ("/usr/lib")

from optparse import OptionParser

import SocketServer
import socket
import os.path
import string
import subprocess
import threading
import logging
import datetime
import signal

import nedm_daq2

##############################################

def logging_config (log_filename, log_level):
  logging.basicConfig (filename=log_filename,
                       format='%(levelname)s\t\t%(threadName)s\t\t%(message)s',
                       level=log_level)
  return
#

##############################################

def py_time ():
  now = datetime.datetime.utcnow ()
  return now.strftime ("%Y-%m-%d  %H:%M:%S.%f\t\t")
#

##############################################

def catch_signal_init ():
  signal.signal (signal.SIGHUP,    catch_signal)
  signal.signal (signal.SIGINT,    catch_signal)
  signal.signal (signal.SIGPIPE,   catch_signal)
  signal.signal (signal.SIGPOLL,   catch_signal)
  signal.signal (signal.SIGPROF,   catch_signal)
  signal.signal (signal.SIGALRM,   catch_signal)
  signal.signal (signal.SIGTERM,   catch_signal)
  signal.signal (signal.SIGUSR1,   catch_signal)
  signal.signal (signal.SIGUSR2,   catch_signal)
  signal.signal (signal.SIGVTALRM, catch_signal)
#

def catch_signal (signum, frame):
  logging.warning (py_time() + " catch signal " + str (signum))
  print ' nEDM FASTER SERVER CLOSED'
  logging.info    (py_time() + " ################   server end   ################")
  return
#

##############################################


class nEDM_Handler(SocketServer.BaseRequestHandler):

  # command names
  HELP               = "help"
  STOP_SERVER        = "stop_server"
  START_NO_STORAGE   = "start_no_storage"
  START_DAQ          = "start_daq"
  STOP_DAQ           = "stop_daq"
  UCN_COUNTS         = "ucn_counts"
  UCN_BLIND          = "ucn_blind"
  UCN_CHECK          = "ucn_check"
  IRIG_TIME          = "irig_time"
  AVALAIBLE_COMMANDS = [HELP, STOP_SERVER, START_NO_STORAGE, START_DAQ, STOP_DAQ, UCN_COUNTS, UCN_BLIND, UCN_CHECK, IRIG_TIME]
  COMMAND_SUMMARY    =                                                                      "\n" + \
      " COMMAND SUMMARY"                                                                  + "\n" + \
      ""                                                                                  + "\n" + \
      "   help"                                                                           + "\n" + \
      "     return this summary."                                                         + "\n" + \
      ""                                                                                  + "\n" + \
      "   start_no_storage"                                                               + "\n" + \
      "     start the acquisition without file storage."                                  + "\n" + \
      "     return 'start_daq ok' or an error message."                                   + "\n" + \
      ""                                                                                  + "\n" + \
      "   start_daq  filename  [user_comment]"                                            + "\n" + \
      "     start the acquisition with filename for storage."                             + "\n" + \
      "     return 'start_daq ok' or an error message."                                   + "\n" + \
      ""                                                                                  + "\n" + \
      "   stop_daq"                                                                       + "\n" + \
      "     stop the acquisition."                                                        + "\n" + \
      "     return 'stop_daq ok' or an error message."                                    + "\n" + \
      ""                                                                                  + "\n" + \
      "   ucn_counts   "                                                                  + "\n" + \
      "     ask Y.Lemiere for arguments, formats & error messages"                        + "\n" + \
      ""                                                                                  + "\n" + \
      "   ucn_blind    "                                                                  + "\n" + \
      "     arguments, formats & error messages : TODO"                                   + "\n" + \
      ""                                                                                  + "\n" + \
      "   ucn_check    "                                                                  + "\n" + \
      "     arguments, formats & error messages : TODO"                                   + "\n" + \
      ""                                                                                  + "\n" + \
      "   irig_time    "                                                                  + "\n" + \
      "     return current tsync time or an error message"                                + "\n"
      #""                                                                                  + "\n" + \
      #"   ucn_counts   filename   start_time   t1   t2  t3  ..."                          + "\n" + \
      #"     ucn counts between start_time and different instants (from filename)."        + "\n" + \
      #"     return ucn count for each requested time or an error message."                + "\n" + \
      #"     return format : 'ucn_counts c1=1123  c2=3421  c3=56734'"                      + "\n" + \
      #""                                                                                  + "\n" + \
      #"   stop_server"                                                                    + "\n" + \
      #"     stop the server and allow the use of the local IHM."                          + "\n" + \
      #"     return 'stop_server ok' or an error message."                                 + "\n"
  #




  def setup (self):
    logging.info (py_time () + " open connexion")
    self.request.settimeout(2.0)
    return
  #


  def finish (self):
    logging.info (py_time () + " close connexion\n")
    return
  #


  def handle (self):
    data = ""
    try:
      data = self.request.recv(1024).replace ('\n', '')
    except socket.timeout:
      logging.error (py_time() + " received nothing - timeout - sent nothing")
      return
    #
    if not data:
      logging.error (py_time() + " received empty message")
      self.request.send (" error : received empty message")
      logging.error (py_time() + " sent error : received empty message")
      return
    #
    try:
      msg  = data.split ()
      cmd  = msg [0]
      args = msg [1:]
      logging.info (py_time () + " received " + data)
      if cmd == nEDM_Handler.HELP:
        self.__help ()
      elif cmd == nEDM_Handler.STOP_SERVER:
        self.__stop_server ()
      elif cmd == nEDM_Handler.START_NO_STORAGE:
        self.__start_no_storage ()
      elif cmd == nEDM_Handler.START_DAQ:
        self.__start_daq (args)
      elif cmd == nEDM_Handler.STOP_DAQ:
        self.__stop_daq ()
      elif cmd == nEDM_Handler.UCN_COUNTS:
        self.__ucn_counts (args)
      elif cmd == nEDM_Handler.UCN_BLIND:
        self.__ucn_blind (args)
      elif cmd == nEDM_Handler.UCN_CHECK:
        self.__ucn_check (args)
      elif cmd == nEDM_Handler.IRIG_TIME:
        self.__irig_time ()
      else:
        self.__unknown_cmd (cmd)
      #
    except Exception, e:
      logging.error (py_time() + " handle exception : " + str (e))
      self.request.send ("handle exception: " + str (e))
    #
    return
  #


  def __help (self):
    self.request.send (nEDM_Handler.COMMAND_SUMMARY)
    logging.info (py_time() + " sent help message")
    return
  #


  def __stop_server (self):
    try:
      self.server.daq.stop ()
    except nedm_daq2.nEDM_DAQ_Exception, e:
      logging.warning (py_time() + " " + str (e))
    #
    ret_msg = "stop_server ok"
    self.request.send (ret_msg)
    logging.info (py_time() + " sent " + ret_msg)
    self.server.shutdown ()
    return
  #


  def __start_no_storage (self):
    try:
      self.server.daq.start ("")
    except Exception, e:
      ret_msg = "start_no_storage error : " + str (e)
      self.request.send (ret_msg)
      logging.error (py_time() + " sent " + ret_msg)
      return
    #
    ##  JH 31-08-15 => Jochen start_daq hook
    p = subprocess.Popen("start_daq", stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    output, errors = p.communicate()
    ##  JH
    ret_msg = "start_no_storage ok" + "\n" + output
    self.request.send (ret_msg)
    logging.info (py_time() + " sent " + ret_msg)
    return
    #
  #


  def __start_daq (self, args):
    n_args = len (args)
    if n_args == 0:
      ret_msg = "start_daq error : no filename specified"
      self.request.send (ret_msg)
      logging.error (py_time() + " sent " + ret_msg)
    else:
      try:
        file_path = os.path.join (self.server.data_dir, args [0])
        if n_args > 1:
          self.server.daq.start (file_path, " ".join (args [1:]))
        else:
          self.server.daq.start (file_path)
        #
      except Exception, e:
        ret_msg = "start_daq error : " + str (e)
        self.request.send (ret_msg)
        logging.error (py_time() + " sent " + ret_msg)
        return
      #
      ##  JH 31-08-15 => Jochen start_daq hook
      ##  JH 05-09-15 => with args 'filename' and 'RunNumber=xxxx'
      if n_args > 1:
          cmd     = ['start_daq', args [0], args [1]]
      else:
          cmd     = ['start_daq', args [0]]
      #
      p              = subprocess.Popen (cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
      output, errors = p.communicate ()
      ret_msg        = "start_daq ok with " + args [0] + "\n" + output
      ##  JH
      self.request.send (ret_msg)
      logging.info (py_time () + " sent " + ret_msg)
      return
    #
  #


  def __stop_daq (self):
    try:
      self.server.daq.stop ()
    except Exception, e:
      ret_msg = "stop_daq error : " + str (e)
      self.request.send (ret_msg)
      logging.error (py_time() + " sent " + ret_msg)
      return
    #
    ##  JH 31-08-15 => Jochen stop_daq hook
    p = subprocess.Popen("stop_daq", stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    output, errors = p.communicate()
    ##  JH
    ret_msg = "stop_daq ok" + "\n" + output
    self.request.send (ret_msg)
    logging.info (py_time() + " sent " + ret_msg)
    return
  #


  def __open_cmd (self, cmd_name, args):
    args.insert (0, cmd_name)
    try:
      rturn = subprocess.Popen (args, stdout=subprocess.PIPE).communicate ()[0]
    except Exception, e:
      ret_msg = cmd_name + " error : " + str (e)
      self.request.send (ret_msg)
      logging.error (py_time() + " sent " + ret_msg)
      return
    #
    self.request.send (rturn)
    logging.info (py_time() + " sent " + rturn.replace ('\n', ' '))
    return
    #
  #


  def __ucn_counts (self, args):
    self.__open_cmd ("ucn_counts", args)
  #


  def __ucn_blind (self, args):
    self.__open_cmd ("ucn_blind", args)
  #


  def __ucn_check (self, args):
    self.__open_cmd ("ucn_check", args)
  #


  def __irig_time (self):
    try:
      rturn = subprocess.Popen (['HW_GetTime', '0'], stdout=subprocess.PIPE).communicate ()[0]
    except Exception, e:
      ret_msg = "irig_time error : " + str (e)
      self.request.send (ret_msg)
      logging.error (py_time() + " sent " + ret_msg)
      return
    #
    self.request.send (rturn)
    logging.info (py_time() + " sent " + rturn)
    return
    #
  #


  def __unknown_cmd (self, cmd):
    ret_msg = "error : unknown command (try 'help')"
    self.request.send (ret_msg)
    logging.error (py_time() + " sent " + ret_msg)
    return
  #

#

##############################################


class nEDM_Server (SocketServer.ThreadingMixIn, SocketServer.TCPServer):

  def __init__(self, server_address=('localhost', 8888), setup_dir = '.', data_dir = '.', without_hardware=False):
    self.data_dir            = data_dir
    self.daq                 = nedm_daq2.nEDM_DAQ2 (setup_dir = setup_dir, without_hardware = without_hardware)
    self.allow_reuse_address = True
    SocketServer.TCPServer.__init__(self, server_address, nEDM_Handler)
    print '\n'
    print ' nEDM FASTER SERVER %s:%d' %(self.server_address [0], self.server_address [1])
    print nEDM_Handler.COMMAND_SUMMARY
    return
  #

#

##############################################

